// InfraRed Receiver routines
// Vishay TSOP4838 38KHz receiver Right leg = +5v, Centre leg = 0v, left leg = output
// your receiver may be different so CHECK THE DATASHEET
// use basic easyPIC7 board at 5V with PIC18F45K22 8MHz xtal + 2x16LCD + IR Rx on C0 with NO pull-up/down
// must turn on UART Tx/Rx RC6/RC7 (SW1-1 and SW2-1) even if not using serial
// pull-down on all PORTD inputs and button jumper to VCC
// connect RS232 terminal 9600 baud, 8 bits, no parity, 1 stop bit, no handshakes
// select options through UART terminal, or using PORTD buttons 0 to 7
// press reset button to return to start screen
// amateur code - use at your own risk - dodgy coding practices
// 38KHz receiver works fine with 40KHz Sony and 36KHz RC5 - just at half the range

// Sharp protocol still under development - needs more work

sbit ir_in at RC0_bit;                      // Infrared receiver input
sbit ir_in_direction at TRISC0_bit;         // Infrared receiver input

// Lcd module connections
sbit LCD_RS at LATA2_bit;
sbit LCD_EN at LATA3_bit;
sbit LCD_D4 at LATA4_bit;
sbit LCD_D5 at LATA5_bit;
sbit LCD_D6 at LATE0_bit;
sbit LCD_D7 at LATE1_bit;

sbit LCD_RS_Direction at TRISA2_bit;
sbit LCD_EN_Direction at TRISA3_bit;
sbit LCD_D4_Direction at TRISA4_bit;
sbit LCD_D5_Direction at TRISA5_bit;
sbit LCD_D6_Direction at TRISE0_bit;
sbit LCD_D7_Direction at TRISE1_bit;
// End Lcd module connections

// constants
#define NSAMPLES 32                // number of IR cycles to read for timing print

// global variables
char fb_string[10];                // just a place to convert numbers to strings
char irnowm;                       // remember current RC5/RC6 mark/space status
char irwas2t;                      // RC5/RC6 have we just had a 2t event?
char irbyte;                       // assemble a byte of IR
char cmd;                          // current command character from UART or PORTD
unsigned int cnt;                  // expands 16 bit counter to 32 bit - not used here
unsigned int retint;               // general purpose integer
signed int temp_bit_s;             // bit timing store
signed int temp_bit_m;             // bit timing store
char keephex[3];                   // store hex value as ascii
unsigned char ir_byte;
char ir_toggle;                    // toggle bit value
unsigned char ir_result[11];

// prototypes see function for explanation
void timing_print(void);
void print_ir_time(unsigned int ir_time);
void sharp_print(void);
void sony_print(void);
int get_mark(void);
int get_space(void);
unsigned char get_sharp_byte(char nbits);
unsigned char get_sony_byte(char nbits);
unsigned char get_rc5_byte(char nbits);
unsigned char get_rc6_byte(char nbits);
unsigned char get_tosh_byte(char nbits);
void getrc5bit(void);
void getrc6bit(void);
void nec_print(void);
unsigned char get_nec_byte(void);
void rc5_print(void);
void rc6_print(void);
void sky_print(void);
void tosh_print(void);
void simple_ir(void);
char get_cmd(void);

void interrupt() {                 // interrepts are here, but serve no function yet
  if (TMR0IF_bit) {                // handle timer 0 overflow
      cnt++;                       // Increment counter - gives extra 2 bytes counter option
      TMR0IF_bit = 0;              // Clear TMR0IF
  }
  if (INT0F_bit){                  // handle interrupt on B0 pin
      INT0F_bit = 0;               // clear interrupt
      if (INTEDG0_bit){
          INTEDG0_bit = 0;         // interrupt on B0 falling edge
      }
      else{
          INTEDG0_bit = 1;         // interrupt on B0 rising edge
      }
  }
}


void main() {
     ANSELA = 0;                   // Configure PORT pins as digital
     ANSELB = 0;                   // Configure PORT pins as digital
     ANSELC = 0;                   // Configure PORT pins as digital
     ANSELD = 0;                   // Configure PORT pins as digital
     ADCON0 = 0;                   // A/D off
     ADCON1 = 0x0f;                // all digital
//     CM1CON = 7;                   // Disable 18F4620 comparator
     CM1CON0 = 0;                  // disable 18F45K22 comparator
     CM2CON0 = 0;                  // disable 18F45K22 comparator
     T0CON = 0x86;                 // timer on 16 bit prescale 128:1
     TMR0L = 96;                   // Timer0 initial value
     TMR0H = 0xff;
     INTCON = 0xB0;                // Enable TMRO and INT interrupts
     cnt = 0;                      // Initialize cnt
     
     LATA = 0;                     // all off
     LATB = 0x80;                  // disable GLCD if fitted
     LATD = 0;                     // all off
     LATC = 0;                     // all off
     LATE = 0;                     // all off
     TRISA = 0xC1;                 // all output except OSC pins
     TRISB = 0x81;                 // all output except b0
     TRISC = 0x01;                 // all output except c0
     TRISD = 0xff;                 // all input
     TRISE = 0;                    // all output
     PWM1_Init(38000);             // 38kHz on RC2
     PWM1_Start;                   // and start PWM running
     PWM1_Set_Duty(80);            // 30%
     PWM2_Init(38000);             // 38kHz on RC1 (or possibly RB3)
     PWM2_Start;                   // and start PWM running
     PWM2_Set_Duty(80);            // 30%
     ir_in_direction = 1;          // IR input
     UART1_Init(9600);             // Initialize UART module at 9600 bps
     Delay_ms(100);                // Wait for UART module to stabilize
     UART1_Write_Text("\r\n\nIR Utilities\r\n\n");

     Delay_ms(1000);                // Wait for LCD to stabilize
     Lcd_Init();                            // Initialize Lcd
     Lcd_Cmd(_LCD_CLEAR);                   // Clear display
     Lcd_Cmd(_LCD_CURSOR_OFF);              // Turn cursor off
     while(1){
        Lcd_Out(1,1, "0LED 1Sh 2T 3NEC");    // Print text to Lcd, 1st row, 1st column
        Lcd_Out(2,1, "4Sony 5RC5 6RC6 ");    // Print text to Lcd, 2nd row, 1st column

        UART1_Write_Text("0 Simple IR test\r\n");
        UART1_Write_Text("1 Sharp Rx (beta)\r\n");
        UART1_Write_Text("2 PT2248/9 Rx\r\n");
        UART1_Write_Text("3 NEC  Rx\r\n");
        UART1_Write_Text("4 Sony Rx\r\n");
        UART1_Write_Text("5 RC5  Rx\r\n");
        UART1_Write_Text("6 RC6 m0, RC6 m6-32 \r\n");
        UART1_Write_Text("7 RC6 mode 6-20 (Sky)\r\n");
        UART1_Write_Text("8 Timing print\r\n");

        cmd = get_cmd();                         // read command from UART or switches
        switch (cmd){
            case '0': simple_ir(); break;
            case '1': sharp_print(); break;
            case '2': tosh_print(); break;
            case '3': nec_print(); break;
            case '4': sony_print(); break;
            case '5': rc5_print(); break;
            case '6': rc6_print(); break;
            case '7': sky_print(); break;
            case '8': timing_print(); break;
            default:  break;
        }
 //       cmd = 0;                               // prevent repeated command
     }
     while(1);              // loop forever
}

// get command character from UART or from PORTD switches
char get_cmd(void){
char retval;

    retval = 0;                             // init
    while(!retval){                         // loop until something found
      if (UART1_Data_Ready()){              // anything received by UART?
          retval = UART1_Read();            // then get the character
      }

      if(PORTD == 0x01) retval='0';         // button D0
      if(PORTD == 0x02) retval='1';         // button D1
      if(PORTD == 0x04) retval='2';         // button D2
      if(PORTD == 0x08) retval='3';         // button D3
      if(PORTD == 0x10) retval='4';         // button D4
      if(PORTD == 0x20) retval='5';         // button D5
      if(PORTD == 0x40) retval='6';         // button D6
      if(PORTD == 0x80) retval='7';         // button D7
    }
    return retval;                          // return ASCII character
}
// simple IR test - just flash LED in response to IR input
void simple_ir(void){
    UART1_Write_Text("\r\nwatch LED on RE0\r\n");
    Lcd_Out(1,1, " Simple IR test ");    // Print text to Lcd, 1st row, 1st column
    Lcd_Out(2,1, "watch LED on RE0");    // Print text to Lcd, 2nd row, 1st column
     while(1){
         if(ir_in){                          // read IR input
              RE0_bit = 0;                   // LED off
         }
         else{
              RE0_bit = 1;                   // LED on
         }
     }
}

// print out timing values to UART
void timing_print(void){
unsigned char ir_bit;
unsigned int ir_mark[NSAMPLES+2];   // mark times
unsigned int ir_space[NSAMPLES+2];  // space times

    UART1_Write_Text("\r\nTiming print\r\n");
    UART1_Write_Text("uS timings are approximate\r\n");
    UART1_Write_Text("Waiting for IR\r\n");
    Lcd_Out(1,1, "timing to RS232 ");     // Print text to Lcd, 1st row, 1st column
    Lcd_Out(2,1, "  9600 8 n 1    ");     // Print text to Lcd, 2nd row, 1st column
    while(ir_in);                            // wait for IR to go low

    for( ir_bit=0 ; ir_bit<(NSAMPLES+1) ; ir_bit++){
        TMR0H = 0;                           // zero timer 0
        TMR0L = 0;
        while(!ir_in);                       // wait for IR to go high
        ir_mark[ir_bit] = TMR0L;             // read timer 0
        ir_mark[ir_bit] +=(TMR0H<<8);
        TMR0H = 0;                           // zero timer 0
        TMR0L = 0;
        while(ir_in);                        // wait for IR to go low
        ir_space[ir_bit] = TMR0L;            // read timer 0
        ir_space[ir_bit] += (TMR0H<<8);
    }

    for( ir_bit=0 ; ir_bit<(NSAMPLES+1) ; ir_bit++){
        IntToStr(ir_bit,fb_string);
        UART1_Write_Text("\r\nbit");
        UART1_Write_Text(fb_string);         // print bit number
        IntToStr(ir_mark[ir_bit],fb_string);
        UART1_Write_Text("   mark");
        UART1_Write_Text(fb_string);         // print mark time
        IntToStr(ir_space[ir_bit],fb_string);
        UART1_Write_Text("    space");
        UART1_Write_Text(fb_string);         // print space time

        UART1_Write_Text("   (");
        print_ir_time(ir_mark[ir_bit]);      // convert T0 value to microseconds
        print_ir_time(ir_space[ir_bit]);
        UART1_Write_Text(" uS)");

    }
    UART1_Write_Text("\r\n\ndone\r\n");
}

// convert timer count to microseconds and print
// not nearly as accurate as the number of digits suggest
void print_ir_time(unsigned int ir_time){
unsigned long time_ul;
unsigned int time_ui;

    time_ul = ir_time;                        // int to long
    time_ul *= 65UL;
    time_ul /= 4UL;
    time_ui = time_ul;                        // long to int
    IntToStr(time_ui,fb_string);              // convert to string
    UART1_Write_Text(fb_string);              // and print it
}

// print unsigned char as hex
void hexout(unsigned char hexval){
int temp_hex;
char hextxt[7];

    temp_hex = hexval;                     // to integer
    IntToHex(temp_hex, hextxt);            // integer to ASCII hex
    UART1_Write_Text(hextxt+2);
    UART1_Write_Text(" ");
    keephex[0] = hextxt[2];                // store ascii hex for later
    keephex[1] = hextxt[3];
    keephex[2] = 0;                        // null terminate
}

// NEC protocol: 1/4 or 1/3 duty cycle 38khz (21 pulses per bit approx)
// 9ms mark then 4.5ms space    header
// 560us mark, 560us space      zero
// 560us mark, 1.69ms space     one
// 4 bytes, LSB first
// 560us mark - long space      stop bit of one
// option 1 = repeat every 110ms (if wanted), but just 9ms mark, 2.25 space, 560us mark (no data)
// option 2 = repeat full message
// 00ff = default address, custom codes available, byte 4 is inverse of byte 3
// byte1 is low byte of address, byte2 is either inverse of byte 1 or high byte

// print out NEC values - 16 bit address, 8 bit command, 8 bit inverted command
void nec_print(void){
signed int temp_ir;
char count_byte;

    UART1_Write_Text("\r\nWaiting for NEC IR\r\n");
    Lcd_Out(1,1, "NEC protocol IR ");    // Print text to Lcd, 1st row, 1st column
    Lcd_Out(2,1, "waiting for IR  ");    // Print text to Lcd, 2nd row, 1st column

    while(1){
        while(ir_in);                         // wait for IR to go low

        temp_ir = get_mark();
        if ((temp_ir > 500) && (temp_ir < 800)){       // expect 9mS mark
            temp_ir = get_space();
            if ((temp_ir > 200) && (temp_ir < 400)){   // expect 4.5mS space
                for(count_byte = 0 ; count_byte < 4 ; count_byte++ ){
                    ir_result[count_byte] = get_nec_byte();
                }
            }
            UART1_Write_Text("\r\nNEC ");
            Lcd_Out(2,1, "NEC            ");    // Print text to Lcd, 2nd row, 1st column
            for(count_byte = 0 ; count_byte < 4 ; count_byte++){
                hexout(ir_result[count_byte]);
                Lcd_Chr(2,(count_byte * 3)+5,keephex[0]);
                Lcd_Chr(2,(count_byte * 3)+6,keephex[1]);
            }
            Delay_ms(10);             // wait for repeat messages to clear
        }
    }
    UART1_Write_Text("\r\n");
}

// get one byte of NEC
unsigned char get_nec_byte(void){
char n;
unsigned char ir_byte;
signed int temp_ir;

    for(n=0 ; n<8 ; n++){
        temp_ir = get_mark();                         // expect ~600uS
        temp_ir = get_space();                        // 500uS = 0, 1.6uS = 1
        ir_byte >>= 1;                                // shift in a zero
        if (temp_ir > 70){                            // check > 900uS
            ir_byte += 0x80;                          // change 0 to a 1 if so
        }
    }
    return ir_byte;                                   // return final byte
}

// get IR mark time (low)
int get_mark(void){
        TMR0H = 0;
        TMR0L = 0;
        while(!ir_in);                       // wait for IR to go high
        LATE.F0 = 0;                         // LED off
        retint = TMR0L;
        retint += (TMR0H<<8);
        return retint;
}

// get IR space time (high)
int get_space(void){
        TMR0H = 0;
        TMR0L = 0;
        while(ir_in);                        // wait for IR to go low
        LATE.F0 = 1;                         // LED on
        retint = TMR0L;
        retint += (TMR0H<<8);
        return retint;
}

// Sharp protocol 38KHz    t = 320uS
// 15 bits followed by G - always at least 2 messages sent with different data
// G = t mark 40ms (inter-message gap)
//
// t mark, 2t space = 0
// t mark, 4t space = 1
// lsb first   5 bits address (0x13), 8 bits command, 2 bits of '10' expansion, or '01' check
// second command byte is inverse of first command byte

// print out Sharp values
void sharp_print(void)
{
signed int temp_ir;

    UART1_Write_Text("\r\nWaiting for Sharp IR\r\n");
    Lcd_Out(1,1, "Sharp protocol  ");    // Print text to Lcd, 1st row, 1st column
    Lcd_Out(2,1, "waiting for IR  ");    // Print text to Lcd, 2nd row, 1st column

    while(1){
        while(ir_in);                         // wait for IR to go low

        ir_result[0] = get_sharp_byte(5);         // 5 bit address
        ir_result[1] = get_sharp_byte(8);         // 8 bit command
        ir_result[2] = get_sharp_byte(2);         // 2 bits extension or check
        ir_result[3] = get_sharp_byte(1);         // 1 bit inter-message gap
        if(ir_result[2]==1){                      // is this a data message?
            UART1_Write_Text("   add=");
            Lcd_Out(2,1, "add=   cmd=     ");    // Print text to Lcd, 2nd row, 1st column
            hexout(ir_result[0]);
            Lcd_Chr(2,5,keephex[0]);
            Lcd_Chr(2,6,keephex[1]);
            UART1_Write_Text(" cmd=");
            hexout(ir_result[1]);
            Lcd_Chr(2,12,keephex[0]);
            Lcd_Chr(2,13,keephex[1]);
            UART1_Write_Text("\r\n");
        }
        Delay_ms(400);                           // allow repeat messages to be lost
    }
}

// get one byte of Sharp
unsigned char get_sharp_byte(char nbits){
char n;
unsigned char ir_byte;
signed int temp_ir;

    ir_byte = 0;                                  // initialise
    for(n=0 ; n<nbits ; n++){
        temp_ir = get_mark();                     // expect ~320uS
        temp_ir = get_space();                    // 680uS = 0, 1680uS = 1
        ir_byte >>= 1;
        if (temp_ir > 70){                        //
            ir_byte += 0x80;
        }
    }
    ir_byte >>= 8-nbits;                          // convert 1/2/5/8 bits to 8
    return ir_byte;
}

// Sony Remote 40kHz 7 bits command (LSB first) then 5 bits address
// there are also 15 bit and 20 bit versions of Sony protocol
//
// 40kHz 7 bits command (LSB first) then 5 bits address
// 2.4ms mark=start
// 0.6sp, 0.6 mark = 0
// 0.6sp, 1.2 mark = 1
//
// tv               address=01
// vcr1             address=02
// vcr2             address=03
// radio/cd         address=04
// laser disc       address=06
// surround sound   address=0c
// tuner            address=0d
// amp              address=10 (theatre / cassette / tuner)
// cd player        address=11
// equaliser        address=12
// HDMI control     address=17
// dvd              address=1a

// print out Sony values
void sony_print(void){
signed int temp_ir;

    UART1_Write_Text("\r\nWaiting for Sony IR\r\n");
    Lcd_Out(1,1, "Sony protocol   ");    // Print text to Lcd, 1st row, 1st column
    Lcd_Out(2,1, "waiting for IR  ");    // Print text to Lcd, 2nd row, 1st column

    while(1){
        while(ir_in);                         // wait for IR to go low

        temp_ir = get_mark();
        if ((temp_ir > 150) && (temp_ir < 250)){       // expect 2.4mS mark
            ir_result[0] = get_sony_byte(7);           // 7 bit command
            ir_result[1] = get_sony_byte(5);           // 5 bit address
            UART1_Write_Text("\r\nSony cmd=");
            Lcd_Out(2,1, "cmd=   addr=    ");          // Print text to Lcd, 2nd row, 1st column
            hexout(ir_result[0]);
            Lcd_Chr(2,5,keephex[0]);
            Lcd_Chr(2,6,keephex[1]);
            UART1_Write_Text(" addr=");
            hexout(ir_result[1]);
            Lcd_Chr(2,13,keephex[0]);
            Lcd_Chr(2,14,keephex[1]);
            Delay_ms(10);             // wait for repeat messages to clear
        }
    }
    UART1_Write_Text("\r\n");
}

// get one byte of Sony
unsigned char get_sony_byte(char nbits){
char n;
unsigned char ir_byte;
signed int temp_ir;

    ir_byte = 0;                                  // initialise
    for(n=0 ; n<nbits ; n++){
        temp_ir = get_space();                    // expect ~510uS
        temp_ir = get_mark();                     // 650uS = 0, 1.2uS = 1
        ir_byte >>= 1;
        if (temp_ir > 60){                        //
            ir_byte += 0x80;
        }
    }
    ir_byte >>= 8-nbits;                          // convert 5/7 bits to 8
    return ir_byte;
}

// RC5 protocol 14 bits 36kHz 1.778ms per high/low bit,
// 24.889ms whole command
// 0=0.889ms mark, 0.889ms space       1=0.889ms space, 0.889ms mark
// start bit = 1     (only second half of bit (mark) is detectable as space is lost in no data period
// start bit = 1     (i.e. two start bits)
// toggle = x        alternates between key presses to distinguish continued press

// print out RC5 values
void rc5_print(void){
signed int temp_ir;
unsigned char rc5_byte1;
unsigned char rc5_byte2;

    UART1_Write_Text("\r\nWaiting for RC5 IR\r\n");
    Lcd_Out(1,1, "RC5 protocol IR ");    // Print text to Lcd, 1st row, 1st column
    Lcd_Out(2,1, "waiting for IR  ");    // Print text to Lcd, 2nd row, 1st column

    while(1){
        while(ir_in);                         // wait for IR to go low

        irwas2t = 0;                                   // init
        irnowm = 0;                                    // init
        temp_ir = get_mark();                          // second half of start bit 1
        if ((temp_ir > 40) && (temp_ir < 80)){         // expect 890uS mark start bit1
            rc5_byte1 = get_rc5_byte(7);               // start bit2 '1' + toggle + 5 bit address
            rc5_byte2 = get_rc5_byte(6);               // 6 bit command
            UART1_Write_Text("\r\nrc5");
            Lcd_Out(2,1, "t=0 ad=   cmd=  ");          // Print text to Lcd, 2nd row, 1st column
            if (rc5_byte1 & 0x20){
                UART1_Write_Text(" t=1");              // toggle bit
                Lcd_Chr(2,3,'1');
            }
            else{
                UART1_Write_Text(" t=0");              // toggle bit
                Lcd_Chr(2,3,'0');
            }
            UART1_Write_Text(" addr=");
            hexout(rc5_byte1 & 0x1F);                  // 5 bits address
            Lcd_Chr(2,8,keephex[0]);                   // recover character stored by hexout function
            Lcd_Chr(2,9,keephex[1]);
            UART1_Write_Text(" cmd=");
            hexout(rc5_byte2);
            Lcd_Chr(2,15,keephex[0]);
            Lcd_Chr(2,16,keephex[1]);
            Delay_ms(100);                              // wait for repeat messages to clear
        }
    }
    UART1_Write_Text("\r\n");
}

// get one byte of RC5
unsigned char get_rc5_byte(char nbits){
char n;

    irbyte = 0;                                  // initialise
    for(n = 0 ; n < nbits ; n++){
        getrc5bit();
    }
    return irbyte;
}

//get a bit from RC5 protocol
void getrc5bit(void){

     if(!irnowm){
          // now a space if here
          if(!irwas2t){
              // now space after 1t mark
              irnowm = 0;                                       // will leave in space state
              temp_bit_s = get_space();                         // get 1st half of bit
              temp_bit_m = get_mark();                          // get 2nd half of bit (+ maybe half of next)
              irwas2t = 0xff;                                   // assume 2t until checked
              if(temp_bit_m < 80) irwas2t = 0;                  // if 1t
              irbyte <<= 1;                                     // rotate in a 0
              irbyte ++;                                        // change to a 1
          }
          else{
              // now space after 2t mark
              irnowm = 0xff;                                    // will leave in mark state
              temp_bit_s = get_space();                         // get 2nd half of bit (+ maybe half of next)
              irwas2t = 0xff;                                   // assume 2t until checked
              if(temp_bit_s < 80) irwas2t = 0;                  // if 1t
              irbyte <<= 1;                                     // rotate in a 0
          }
     }
     else{
          // now a mark if here
          if(!irwas2t){
              // now mark after 1t space
              irnowm = 0xff;                                    // will leave in mark state
              temp_bit_m = get_mark();                          // get 1st half of bit
              temp_bit_s = get_space();                         // get 2nd half of bit (+ maybe half of next)
              irwas2t = 0xff;                                   // assume 2t until checked
              if(temp_bit_s < 80) irwas2t = 0;                  // if 1t
              irbyte <<= 1;                                     // rotate in a 0
          }
          else{
              // now mark after 2t space
              irnowm = 0;                                       // will leave in space state
              temp_bit_m = get_mark();                          // get 2nd half of bit (+ maybe half of next)
              irwas2t = 0xff;                                   // assume 2t until checked
              if(temp_bit_m < 80) irwas2t = 0;                  // if 1t
              irbyte <<= 1;                                     // rotate in a 0
              irbyte ++;                                        // change to a 1
          }
     }
}

// print out RC6 values
// 36kHz 25% to 50% duty cycle (16 pulses for 50%, 8 pulses for 25%)
//  header:
//    2.66ms mark, 889uS space = leader (6t mark, 2t space)
//    3 mode bits = 0 or 6           where 444uS space 444us mark = 0, 444us mark 444us space = 1
//    trailer bit double length      where 889uS space 889us mark = 0, 889us mark 889us space = 1
//  8 bits control       msb first
//  8 bits information   msb first
//  6t signal free time

void rc6_print(void){
signed int temp_ir;
unsigned char rc6_mode;
char leader6;

    UART1_Write_Text("\r\nWaiting for RC6 IR\r\n");
    Lcd_Out(1,1, "RC6m0, RC6m6-32 ");    // Print text to Lcd, 1st row, 1st column
    Lcd_Out(2,1, "waiting for IR  ");    // Print text to Lcd, 2nd row, 1st column

    while(1){
        while(ir_in);                         // wait for IR to go low

        // wait for leader 6t mark (2.667mS) 2t space (889uS)
        leader6 = 0;        // init
        
        while(!leader6){
            temp_ir = get_mark();                      // get leader
            if ((temp_ir > 140) && (temp_ir < 200)){   // expect 2.66mS mark
              temp_ir = get_space();
              if ((temp_ir > 40) && (temp_ir < 60)){   // expect 889uS space
                leader6 = 1;                           // leader found - allow exit loop
              }
            }
        }

        irwas2t = 0;                                   // init
        irnowm = 0xff;                                 // init
        rc6_mode = get_rc6_byte(4);                    // start bit2 '1' + 3 bits mode
        rc6_mode &= 0x07;                              // remove start bit '1' to leave 3 bits mode
        
        // handle double-size toggle bit
        if (irwas2t){                                  // actually detecting 3t, single toggle bit is double size
            ir_toggle = 1;                             // have found a trailer / toggle bit of 1
            irnowm = 0;                                // leave in space state
            temp_ir = get_space();
            irwas2t = 0xff;                            // assume 2t until checked
            if(temp_bit_s < 70){
                irwas2t = 0;                           // if 1t
            }
        }
        else{
            ir_toggle = 0;                             // have found a trailer / toggle bit of 0
            irnowm = 0;                                // leave in space state
            temp_ir = get_space();                     // skip over the space
            temp_ir = get_mark();                      // read the mark
            irwas2t = 0xff;                            // assume 2t until checked
            if(temp_bit_m < 70){
                irwas2t = 0;                           // if 1t
            }
        }
        
        if (rc6_mode == 0){
            // handle RC6 mode 0
            ir_result[1] = get_rc6_byte(8);            // control byte
            ir_result[2] = get_rc6_byte(8);            // information byte

            UART1_Write_Text("rc6 mode 0");
            Lcd_Out(1,1, "RC6 mode 0      ");          // Print text to Lcd, 1st row, 1st column
            Lcd_Out(2,1, "t=0 ct=   inf=  ");          // Print text to Lcd, 2nd row, 1st column
            if (ir_toggle){
                UART1_Write_Text(" t=1");              // toggle bit
                Lcd_Chr(2,3,'1');
            }
            else{
                UART1_Write_Text(" t=0");              // toggle bit
                Lcd_Chr(2,3,'0');
            }
            UART1_Write_Text(" ctrl=");
            hexout(ir_result[1]);                      // 8 bits control byte
            Lcd_Chr(2,8,keephex[0]);                   // recover character stored by hexout function
            Lcd_Chr(2,9,keephex[1]);
            UART1_Write_Text(" inf=");
            hexout(ir_result[2]);
            Lcd_Chr(2,15,keephex[0]);
            Lcd_Chr(2,16,keephex[1]);
            Delay_ms(10);                              // wait for repeat messages to clear
        }

        if (rc6_mode == 6){
            // handle RC6 mode 6
            ir_result[1] = get_rc6_byte(8);               // full byte
            ir_result[2] = get_rc6_byte(8);               // full byte
            ir_result[3] = get_rc6_byte(8);               // full byte
            ir_result[4] = get_rc6_byte(7);               // only 7 bits of byte
            // last bit fiddly, as don't want to have to wait for new start bit before detecting last bit
            ir_result[3] <<= 1;                           // leave room for last bit
            if (ir_result[3] & 0x02){                     // penultimate bit was 1
                if (!irwas2t){
                    ir_result[3]++;                       // last bit must be 1
                }
                get_mark();
            }
            else{                                         // penultimate bit was 0
                if (irwas2t){
                    ir_result[3]++;                       // last bit must be 1
                }
                else{
                    get_mark;
                }
            }

            UART1_Write_Text("rc6 mode 6-32");
            Lcd_Out(1,1, "RC6 mode 6-32   ");      // Print text to Lcd, 2nd row, 1st column
            Lcd_Out(2,1, "t=0             ");      // Print text to Lcd, 2nd row, 1st column
            if (ir_toggle){
                UART1_Write_Text(" t=1");              // toggle bit
                Lcd_Chr(2,3,'1');
            }
            else{
                UART1_Write_Text(" t=0");              // toggle bit
                Lcd_Chr(2,3,'0');
            }
            UART1_Write_Text(" ");
            hexout(ir_result[1]);                         // 4 bits
            Lcd_Chr(2,8,keephex[0]);
            Lcd_Chr(2,9,keephex[1]);
            hexout(ir_result[2]);
            Lcd_Chr(2,10,keephex[0]);
            Lcd_Chr(2,11,keephex[1]);
            hexout(ir_result[3]);
            Lcd_Chr(2,12,keephex[0]);
            Lcd_Chr(2,13,keephex[1]);
            hexout(ir_result[4]);
            Lcd_Chr(2,14,keephex[0]);
            Lcd_Chr(2,15,keephex[1]);
            Delay_ms(10);                              // wait for repeat messages to clear
        }
        UART1_Write_Text("\r\n");
    }
}

// get one byte of RC6
unsigned char get_rc6_byte(char nbits){
char n;

    irbyte = 0;                                  // initialise
    for(n = 0 ; n < nbits ; n++){
        getrc6bit();
    }
    return irbyte;
}

//get a bit from RC6 protocol
void getrc6bit(void){

     if(!irnowm){
         // now a space if here
          if(!irwas2t){
              // now space after 1t mark
              irnowm = 0;                                       // will leave in space state
              temp_bit_s = get_space();                         // get 1st half of bit
              temp_bit_m = get_mark();                          // get 2nd half of bit (+ maybe half of next)
              irwas2t = 0xff;                                   // assume 2t until checked
              if(temp_bit_m < 40) irwas2t = 0;                  // if 1t
              irbyte <<= 1;                                     // rotate in a 0
          }
          else{
              // now space after 2t mark
              irnowm = 0xff;                                    // will leave in mark state
              temp_bit_s = get_space();                         // get 2nd half of bit (+ maybe half of next)
              irwas2t = 0xff;                                   // assume 2t until checked
              if(temp_bit_s < 40) irwas2t = 0;                  // if 1t
              irbyte <<= 1;                                     // rotate in a 0
              irbyte ++;                                        // change to a 1
          }
     }
     else{
          // now a mark if here
          if(!irwas2t){
              // now mark after 1t space
              irnowm = 0xff;                                    // will leave in mark state
              temp_bit_m = get_mark();                          // get 1st half of bit
              temp_bit_s = get_space();                         // get 2nd half of bit (+ maybe half of next)
              irwas2t = 0xff;                                   // assume 2t until checked
              if(temp_bit_s < 40) irwas2t = 0;                  // if 1t
              irbyte <<= 1;                                     // rotate in a 0
              irbyte ++;                                        // change to a 1
          }
          else{
              // now mark after 2t space
              irnowm = 0;                                       // will leave in space state
              temp_bit_m = get_mark();                          // get 2nd half of bit (+ maybe half of next)
              irwas2t = 0xff;                                   // assume 2t until checked
              if(temp_bit_m < 40) irwas2t = 0;                  // if 1t
              irbyte <<= 1;                                     // rotate in a 0
          }
     }
}

// handle RC6 mode 6-20 (UK Sky remote)
void sky_print(void){
signed int temp_ir;
unsigned char rc6_mode;
char leader6;

    UART1_Write_Text("\r\nWaiting for RC6 IR\r\n");
    Lcd_Out(1,1, "RC6 mode6-20 Sky");    // Print text to Lcd, 1st row, 1st column
    Lcd_Out(2,1, "waiting for IR  ");    // Print text to Lcd, 2nd row, 1st column

    while(1){
        while(ir_in);                         // wait for IR to go low

        // wait for leader 6t mark (2.667mS) 2t space (889uS)
        leader6 = 0;        // init

        while(!leader6){
            temp_ir = get_mark();                      // get leader
            if ((temp_ir > 140) && (temp_ir < 200)){   // expect 2.66mS mark
              temp_ir = get_space();
              if ((temp_ir > 40) && (temp_ir < 60)){   // expect 889uS space
                leader6 = 1;                           // leader found - allow exit loop
              }
            }
        }

        irwas2t = 0;                                   // init
        irnowm = 0xff;                                 // init
        rc6_mode = get_rc6_byte(4);                    // start bit2 '1' + 3 bits mode
        rc6_mode &= 0x07;                              // remove start bit '1' to leave 3 bits mode

        // handle double-size toggle bit
        if (irwas2t){                                  // actually detecting 3t, single toggle bit is double size
            ir_toggle = 1;                             // have found a trailer / toggle bit of 1
            irnowm = 0;                                // leave in mark state
            temp_ir = get_space();
            irwas2t = 0xff;                            // assume 2t until checked
            if(temp_bit_s < 70){
                irwas2t = 0;                           // if 1t
            }
        }
        else{
            ir_toggle = 0;                             // have found a trailer / toggle bit of 0
            irnowm = 0;                                // leave in mark state
            temp_ir = get_space();                     // skip over the mark
            temp_ir = get_mark();                      // read the space
            irwas2t = 0xff;                            // assume 2t until checked
            if(temp_bit_m < 70){
                irwas2t = 0;                           // if 1t
            }
        }

        if (rc6_mode == 0){
            // handle RC6 mode 0
            ir_result[1] = get_rc6_byte(8);               // control byte
            ir_result[2] = get_rc6_byte(8);               // information byte

            UART1_Write_Text("rc6 mode 0");
            Lcd_Out(1,1, "RC6 mode 0      ");      // Print text to Lcd, 2nd row, 1st column
            Lcd_Out(2,1, "t=0 ct=   inf=  ");      // Print text to Lcd, 2nd row, 1st column
            if (ir_toggle){
                UART1_Write_Text(" t=1");              // toggle bit
                Lcd_Chr(2,3,'1');
            }
            else{
                UART1_Write_Text(" t=0");              // toggle bit
                Lcd_Chr(2,3,'0');
            }
            UART1_Write_Text(" ctrl=");
            hexout(ir_result[1]);                         // 8 bits control byte
            Lcd_Chr(2,8,keephex[0]);               // recover character stored by hexout function
            Lcd_Chr(2,9,keephex[1]);
            UART1_Write_Text(" inf=");
            hexout(ir_result[2]);
            Lcd_Chr(2,15,keephex[0]);
            Lcd_Chr(2,16,keephex[1]);
            Delay_ms(10);                              // wait for repeat messages to clear
        }

        if (rc6_mode == 6){
            // handle RC6 mode 6
            ir_result[1] = get_rc6_byte(4);               // first nibble
            ir_result[2] = get_rc6_byte(8);               // full byte
            ir_result[3] = get_rc6_byte(7);               // only 7 bits of byte
            // last bit fiddly, as don't want to have to wait for new start bit before detecting last bit
            ir_result[3] <<= 1;                           // leave room for last bit
            if (ir_result[3] & 0x02){                     // penultimate bit was 1
                if (!irwas2t){
                    ir_result[3]++;                       // last bit must be 1
                }
                get_mark();
            }
            else{                                      // penultimate bit was 0
                if (irwas2t){
                    ir_result[3]++;                       // last bit must be 1
                }
                else{
                    get_mark;
                }
            }

            UART1_Write_Text("RC6 mode 6-20");
            Lcd_Out(1,1, "RC6 mode6-20 Sky");      // Print text to Lcd, 2nd row, 1st column
            Lcd_Out(2,1, "t=0             ");      // Print text to Lcd, 2nd row, 1st column
            if (ir_toggle){
                UART1_Write_Text(" t=1");              // toggle bit
                Lcd_Chr(2,3,'1');
            }
            else{
                UART1_Write_Text(" t=0");              // toggle bit
                Lcd_Chr(2,3,'0');
            }
            UART1_Write_Text(" ");
            hexout(ir_result[1]);                         // 4 bits
            Lcd_Chr(2,9,keephex[1]);
            hexout(ir_result[2]);
            Lcd_Chr(2,10,keephex[0]);
            Lcd_Chr(2,11,keephex[1]);
            hexout(ir_result[3]);
            Lcd_Chr(2,12,keephex[0]);
            Lcd_Chr(2,13,keephex[1]);
            Delay_ms(10);                              // wait for repeat messages to clear
        }
        UART1_Write_Text("\r\n");
    }
}

// PT2248 protocol - my guess at how it works - see PT2248 datasheet
// t = 440uS
// 3t mark, t space = 1 ?
// t mark, 3t space = 0 ?
// msb first ?      19ms total message time for 48t
// 110BA0xxxxx0 where A=row A, B=row B,xxxxx is column

// toshiba protocol as used by PT2248 tx chip and PT2249 rx chip
void tosh_print(void){
signed int temp_ir;
unsigned char tosh_result[4];

    UART1_Write_Text("\r\nWaiting for Toshiba IR\r\n");
    Lcd_Out(1,1, "Toshiba protocol");    // Print text to Lcd, 1st row, 1st column
    Lcd_Out(2,1, "waiting for IR  ");    // Print text to Lcd, 2nd row, 1st column

    while(1){
        while(ir_in);                              // wait for IR to go low
        irnowm = 1;                                // initilise
        tosh_result[0] = get_tosh_byte(3);         // 3 bit code
        tosh_result[1] = get_tosh_byte(3);         // 3 bit header
        tosh_result[2] = get_tosh_byte(6);         // 6 bit data
        UART1_Write_Text("\r\nToshiba code=");
        Lcd_Out(2,1, "code=  h=  d=   ");      // Print text to Lcd, 2nd row, 1st column
        hexout(tosh_result[0]);
        Lcd_Chr(2,6,keephex[1]);
        UART1_Write_Text(" header=");
        hexout(tosh_result[1]);
        Lcd_Chr(2,10,keephex[1]);
        UART1_Write_Text(" data=");
        hexout(tosh_result[2]);
        Lcd_Chr(2,14,keephex[0]);
        Lcd_Chr(2,15,keephex[1]);
        Delay_ms(1);             // wait for repeat messages to clear
    }
    UART1_Write_Text("\r\n");
}

// get one byte of Toshiba protocol
// looks only for long marks or spaces - just ignores the short ones
unsigned char get_tosh_byte(char nbits){
char n;
unsigned char ir_byte;
signed int temp_ir;

    ir_byte = 0;                                  // initialise
    for(n=0 ; n<nbits ; n++){
        if (irnowm){
            temp_ir = get_space();                // keep looking for long pulse mark or space
            irnowm = 0;                           // keep pace
            if (temp_ir < 60) {
                temp_ir = get_mark();
                irnowm = 1;                       // keep pace
                if (temp_ir < 60) {
                    temp_ir = get_space();
                    irnowm = 0;                   // keep pace
                }
            }
        }
        else{
            temp_ir = get_mark();
            irnowm = 1;                           // keep pace
            if (temp_ir < 60) {
                temp_ir = get_space();
                irnowm = 0;                       // keep pace
                if (temp_ir < 60) {
                    temp_ir = get_mark();
                    irnowm = 1;                   // keep pace
                }
            }

        }
        
        
        ir_byte <<= 1;                            // shift in a zero
        if (irnowm){
            ir_byte += 0x01;                      // change to a 1
        }
    }
    return ir_byte;
}