// Infrared Transmit routines to simulate IR TV remote control
// I have deliberately tried to keep this code simple - so it is easy to copy and adapt
//
// For EasyPIC7 board with PIC18F45K22 fitted
// 8MHz crystal with 4xPLL enabled to give 32MHz clock
// 9600,8,n,1 terminal connection is optional
// Turn on RX and TX UART switches (SW1.1 and SW2.1).
// Put RX and TX UART jumpers (J3 and J4) in RS-232 or USB position
// Connect IR LED with 33 ohm resistor in series to RC2 for low power operation
// Connect IR driver circuit to RC2 for higher power (and thus greater range) - high=IR on
// IR Tx circuit is included in the project directory
// select pull-down on all PORTD inputs and set button press level VCC (high)
// amateeur code at users risk. Not to be used as an example of good programming
// author accepts no responsibility for anything of any kind at any time, ever
// some code may be based on MikroElectronica examples and permission may be needed if selling this code on
// author gives permision for free use for any purpose of any sections of code not copyright MikroElectronica
//
// see http://www.sbprojects.com/knowledge/ir/index.php for good information on domestic IR remote protocols
//
//   Sharp, Toshiba, NEC, Sony, RC5, RC6 mode 0, RC6 mode 6A-32 protocols supported
// coding is finished, but still requires testing and probably bug fixes needed too
// Sharp protocol is totally untested, as I have no Sharp device to test on

// prototypes
void send_tosh(void);
void send_nec(void);
void send_sharp(void);
void send_sony(void);
void send_rc5(void);
void send_rc6m0(void);
void send_rc6m6(void);
void set_36(void);
void set_38(void);
void set_40(void);
void tosh_0(void);
void tosh_1(void);
void tosh_bit(char rc5_bitval);
void rc5_0(void);
void rc5_1(void);
void ir_rc5_bit(char rc5_bitval);
void rc6_0(void);
void rc6_1(void);
void ir_rc6_bit(char rc6_bitval);
void rc6_double_0(void);
void rc6_double_1(void);
void rc6_double_bit(char bitval);

// global variables
unsigned char select;                    // store PORTD switch selections
unsigned char ir_byte[5];                // store IR bytes 1 to 4 ready for transmission - byte0 is spare
unsigned char toggle;                    // zero or non zero alternating toggle bit

void main() {
    ANSELC = 0;                          // Configure PORTC pins as digital
    TRISC = 0;                           // all output
    ANSELD = 0;                          // Configure PORTD pins as digital
    TRISD = 0xff;                        // all input
    toggle = 0;                          // initialise (not really needed)

    UART1_Init(9600);                    // Initialize UART module at 9600 bps
    Delay_ms(100);                       // Wait for UART module to stabilize

    UART1_Write_Text("\r\n\nEasyPIC7 IR Transmit\r\n\n");
    UART1_Write_Text("Set pull-downs on all PORT D inputs\r\n");
    UART1_Write_Text("Set button press level to high\r\n");
    UART1_Write_Text("Connect IR LED with 33 Ohm resitor in series onto pin C2\r\n");
    UART1_Write_Text("Take a Button on PORTD high as below:\r\n\n");
    UART1_Write_Text("0 Simple Test 20mS mark, 100mS space 38KHz\r\n");
    UART1_Write_Text("1 Sharp 0x13 0xa4\r\n");
    UART1_Write_Text("2 PT2248 Toshiba\r\n");
    UART1_Write_Text("3 NEC 1234 56 (A9)\r\n");
    UART1_Write_Text("4 Turn Sony TV on/off\r\n");
    UART1_Write_Text("5 RC5\r\n");
    UART1_Write_Text("6 RC6 mode 0 12 34\r\n");
    UART1_Write_Text("7 RC6 mode 6A-32\r\n");

    PWM1_Init(38000);                    // Initialize PWM1 (and PWM2) module at 38KHz
    PWM1_Set_Duty(80);                   // approx 1/3 duty cycle is typical for IR

    while(1){                            // main loop
        select = PORTD;                  // read 8 buttons on PORTD
        
        if(select == 1){                 // button RD0 pressed (and no others)
            set_38();                    // change IR frequency
            PWM1_Start();
            Delay_ms(20);
            PWM1_Stop();
            Delay_ms(100);
        }
        if(select == 2){                 // button RD1 pressed (and no others)
            ir_byte[1]=0x13;             // Sharp 5 bit address
            ir_byte[2]=0xa4;             // Sharp 7 bit command
            send_sharp();                // and send it...
        }
        if(select == 4){                 // button RD2 pressed (and no others)
            ir_byte[1]=4;                // Toshiba 3 bit code (address)
            ir_byte[2]=4;                // one H and two S bits
            ir_byte[3]=8;                // 6 key-code bits
            send_tosh();                 // and send it...
        }
        if(select == 8){                 // button RD3 pressed (and no others)
            ir_byte[1]=0x12;             // NEC address
            ir_byte[2]=0x34;             // NEC address
            ir_byte[3]=0x56;             // NEC data
            send_nec();                  // and send it...
        }
        if(select == 16){                // button RD4 pressed (and no others)
            ir_byte[1]=0x15;             // Sony 7 bit command (0x15=power)
            ir_byte[2]=1;                // Sony 5 bit address (1=TV)
            send_sony();                 // and send it...
            send_sony();                 // later TVs need two messages (spaced 26 mS apart)
        }
        if(select == 32){                // button RD5 pressed (and no others)
            ir_byte[1]=0x00;             // RC5 5 bit address (0=TV)
            ir_byte[2]=0x0d;             // RC5 6 bit command (13=mute)
            send_rc5();                  // and send it...
        }
        if(select == 64){                // button RD6 pressed (and no others)
            ir_byte[1]=0x12;             // RC6 8 bit control
            ir_byte[2]=0x34;             // RC6 8 bit information
            send_rc6m0();                // and send it...
        }
        if(select == 128){               // button RD7 pressed (and no others)
            ir_byte[1]=0x00;             // RC6
            ir_byte[2]=0x22;             // RC6
            ir_byte[3]=0x10;             // RC6
            ir_byte[4]=0x0c;             // RC6
            send_rc6m6();                // and send it...
        }
        select = 0;
    }

    while (1);                           // Endless loop
}


// set IR frequency to 36 KHz (RC5 and RC6 protocols)
void set_36(){
     PR2 = 220;
}

// set IR frequency to 38 KHz (NEC protocol)
void set_38(){
     PR2 = 210;
}

// set IR frequency to 40 KHz (Sony protocol)
void set_40(){
     PR2 = 200;
}

// send a single IR message in Toshiba protocol PT2248
// put 3 bit code into ir_byte[1], one H and two S bits into ir_byte[2], 6 key-code bits into ir_byte[3] before calling
// This is not really Toshiba protocol at all - I just chose it for convenient naming after finding it used by a Toshiba remote
void send_tosh(void){
char bitnum;

    set_38();                                  // 38 KHz IR
    Delay_ms(100);                             // ensure some no-data time

    for(bitnum=0 ; bitnum < 3 ; bitnum++){     // 3 bit address
        tosh_bit(ir_byte[1] >> (2-bitnum));     // MSB sent first
    }
    for(bitnum=0 ; bitnum < 3 ; bitnum++){     // H bit and 2 S bits
        tosh_bit(ir_byte[2] >> (2-bitnum));     // MSB sent first
    }
    for(bitnum=0 ; bitnum < 6 ; bitnum++){     // 6 bit key-code
        tosh_bit(ir_byte[3] >> (5-bitnum));
    }
    PWM1_Stop();
    Delay_ms(100);                             // ensure some no-data time
}


// send a single IR message in RC6 mode 0 protocol
// put 8 bit control into ir_byte[1] and 8 bit information into ir_byte[2] before calling
void send_rc6m0(void){
char bitnum;

    set_36();                                  // 36 KHz IR
    Delay_ms(100);                             // ensure some no-data time
    PWM1_Start();                              // mark
    Delay_us(2666);                            // 2.666mS mark leader
    PWM1_Stop();                               // space
    Delay_us(889);                             // 889uS space ends leader
    rc6_1();                                   // start bit of one
    rc6_0();                                   // first mode bit of zero
    rc6_0();                                   // second mode bit of zer0
    rc6_0();                                   // third mode bit of zero
    rc6_double_bit(toggle);                    // send double size toggle bit
    toggle++;                                  // toggle ready for next time

    for(bitnum=0 ; bitnum < 8 ; bitnum++){     // 8 bit address
        ir_rc6_bit(ir_byte[1] >> (7-bitnum));     // MSB sent first
    }
    for(bitnum=0 ; bitnum < 8 ; bitnum++){     // 8 bit command
        ir_rc6_bit(ir_byte[2] >> (7-bitnum));
    }
    PWM1_Stop();
    Delay_ms(100);                             // ensure some no-data time
}

// send a single IR message in RC6 mode 6 protocol
// put 8 bit dat into ir_byte[1] to ir_byte[4] before calling
void send_rc6m6(void){
char bitnum;

    set_36();                                  // 36 KHz IR
    Delay_ms(100);                             // ensure some no-data time
    PWM1_Start();                              // mark
    Delay_us(2666);                            // 2.666mS mark leader
    PWM1_Stop();                               // space
    Delay_us(889);                             // 889uS space ends leader
    rc6_1();                                   // start bit of one
    rc6_1();                                   // first mode bit of one
    rc6_1();                                   // second mode bit of one
    rc6_0();                                   // third mode bit of zero
    rc6_double_bit(toggle);                    // send double size toggle bit
    toggle++;                                  // toggle ready for next time

    for(bitnum=0 ; bitnum < 8 ; bitnum++){     // 8 bits
        ir_rc6_bit(ir_byte[1] >> (7-bitnum));     // MSB sent first
    }
    for(bitnum=0 ; bitnum < 8 ; bitnum++){     // 8 bits
        ir_rc6_bit(ir_byte[2] >> (7-bitnum));
    }
    for(bitnum=0 ; bitnum < 8 ; bitnum++){     // 8 bits
        ir_rc6_bit(ir_byte[3] >> (7-bitnum));     // MSB sent first
    }
    for(bitnum=0 ; bitnum < 8 ; bitnum++){     // 8 bits
        ir_rc6_bit(ir_byte[4] >> (7-bitnum));
    }
    PWM1_Stop();
    Delay_ms(100);                             // ensure some no-data time
}

// send a single IR message in RC5 protocol
// put 5 bit address into ir_byte[1] and 6 bit command into ir_byte[2] before calling
void send_rc5(void){
char bitnum;

    set_36();                                  // 36 KHz IR
    Delay_ms(100);                             // ensure some no-data time
    rc5_1();                                   // first of two start bits of one
    rc5_1();                                   // second of two start bits of one
    ir_rc5_bit(toggle);                           // send toggle bit
    toggle++;                                  // toggle ready for next time
     
    for(bitnum=0 ; bitnum < 5 ; bitnum++){     // 5 bit address
        ir_rc5_bit(ir_byte[1] >> (4-bitnum));     // MSB sent first
    }
    for(bitnum=0 ; bitnum < 6 ; bitnum++){     // 6 bit command
        ir_rc5_bit(ir_byte[2] >> (5-bitnum));
    }
    PWM1_Stop();
    Delay_ms(100);                             // ensure some no-data time
}

// send a single Toshiba bit of 0    889uS mark, 889uS space
void tosh_0(void){
    PWM1_Start();                        // mark a
    Delay_us(422);
    PWM1_Stop();                         // space 3a
    Delay_us(1266);
}

// send a single Toshiba bit of 1    889uS space, 889uS mark
void tosh_1(void){
    PWM1_Start();                        // mark 3a
    Delay_us(1266);
    PWM1_Stop();                         // space a
    Delay_us(422);
}

// send 0 or 1 depending on value of tosh_bitval
void tosh_bit(char tosh_bitval){

    if(tosh_bitval & 1){
        tosh_1();
    }
    else{
        tosh_0();
    }
}


// send a single RC5 bit of 0    889uS mark, 889uS space
void rc5_0(void){
    PWM1_Start();                        // mark
    Delay_us(889);
    PWM1_Stop();                         // space
    Delay_us(889);
}

// send a single RC5 bit of 1    889uS space, 889uS mark
void rc5_1(void){
    PWM1_Stop();                         // space
    Delay_us(889);
    PWM1_Start();                        // mark
    Delay_us(889);
}

// send 0 or 1 depending on value of rc5_bitval
void ir_rc5_bit(char rc5_bitval){

    if(rc5_bitval & 1){
        rc5_1();
    }
    else{
        rc5_0();
    }
}

// send double length toggle bit 0 or 1 depending on value of bitval
void rc6_double_bit(char bitval){

    if(bitval & 1){
        rc6_double_1();
    }
    else{
        rc6_double_0();
    }
}

// send a single RC6 bit (double length) of 1    889uS mark, 889uS space
void rc6_double_1(void){
    PWM1_Start();                        // mark
    Delay_us(889);
    PWM1_Stop();                         // space
    Delay_us(889);
}

// send a single RC6 bit (double length) of 0    889uS space, 889uS mark
void rc6_double_0(void){
    PWM1_Stop();                         // space
    Delay_us(889);
    PWM1_Start();                        // mark
    Delay_us(889);
}

// send a single RC6 bit of 1    444uS mark, 444uS space
void rc6_1(void){
    PWM1_Start();                        // mark
    Delay_us(444);
    PWM1_Stop();                         // space
    Delay_us(444);
}

// send a single RC6 bit of 0    444uS space, 444uS mark
void rc6_0(void){
    PWM1_Stop();                         // space
    Delay_us(444);
    PWM1_Start();                        // mark
    Delay_us(444);
}

// send 0 or 1 depending on value of rc5_bitval
void ir_rc6_bit(char rc6_bitval){

    if(rc6_bitval & 1){
        rc6_1();
    }
    else{
        rc6_0();
    }
}

// send a single IR message in NEC 32-bit protocol
// put address into ir_byte[1], ir_byte[2] and command into ir_byte[3] before calling
void send_nec(void){
char bitnum;
char bytenum;

     set_38();                                       // 38 KHz IR
     ir_byte[4] = ~ir_byte[3];                       // byte 4 is inverse of byte 3
     Delay_ms(100);                                  // precaution to be certain of some dead time before message start
     PWM1_Start();                                   // mark
     Delay_us(9000);                                 // 9mS mark start bit
     PWM1_Stop();                                    // space
     Delay_us(4500);                                 // 4.5mS space ends start bit
     for(bytenum = 1; bytenum < 5; bytenum++){       // 4 bytes to send
         for(bitnum = 0; bitnum < 8; bitnum++){      // 8 bits LSB first
             PWM1_Start();                           // mark
             Delay_us(560);                          // 560uS mark for '0' or '1'
             PWM1_Stop();                            // space
             if(ir_byte[bytenum] & (1 << bitnum)){   // test bit <bitnum>
                 Delay_us(1680);                     // 1.68mS space for a '1'
             }
             else{
                 Delay_us(560);                      // 560uS space for a '0'
             }


         }
     }
     PWM1_Start();                                   // mark
     Delay_us(560);                                  // stop bit of '1'
     PWM1_Stop();                                    // space
     Delay_us(1680);                                 // stop bit of 1
     Delay_ms(100);                                  // precaution to be certain of some dead time after message ends
}

// send a single IR message in Sony 12-bit protocol
// put 7 bit command into ir_byte[1] and 5 bit address into ir_byte[2] before calling
// call twice for modern Sony TVs, as they seem to respond only to double messages
//
// some Sony remotes can be configured to act as 'service remotes',
// this changes one of the buttons to become a 'test' button,
// pressing it once displays 'T' on the screen, pressing it twice displays 'TT' and enters test mode
//  - pressing 'Menu' at this point displays the service menu. (information button on my TV)
// In order to make yourself a 'service remote' you just need to send the Device ID '1' and the command '127'.
void send_sony(void){
char bitnum;

     set_40();                                   // 40 KHz IR
     PWM1_Start();                               // mark
     Delay_us(2400);                             // 2.4mS mark start bit
     PWM1_Stop();                                // space
     Delay_us(600);                              // 600uS space ends start bit
     for(bitnum = 0; bitnum < 7; bitnum++){      // 7 bit command LSB first
         PWM1_Start();                           // mark
         if(ir_byte[1] & (1 << bitnum)){         // test bit <bitnum>
             Delay_us(1200);                     // 1.2mS mark for a '1'
         }
         else{
             Delay_us(600);                      // 600uS mark for a '0'
         }
         PWM1_Stop();                            // space
         Delay_us(600);                          // 600uS space for '0' or '1'
     }
     for(bitnum = 0; bitnum < 5; bitnum++){      // 5 bit address LSB first
         PWM1_Start();                           // mark
         if(ir_byte[2] & (1 << bitnum)){        // test bit <bitnum>
             Delay_us(1200);                     // 1.2mS mark for a '1'
         }
         else{
             Delay_us(600);                      // 600uS mark for a '0'
         }
         PWM1_Stop();                            // space
         Delay_us(600);                          // 600uS space for '0' or '1'
     }
     Delay_ms(26);                               // 26 millisecond inter-message gap
}

// sharp protocol - totally untested - as I have no Sharp device to test on
void send_sharp(void){
char bitnum;

     set_38();                                   // 38 KHz IR
     for(bitnum = 0; bitnum < 5; bitnum++){      // 5 bit address LSB first
         PWM1_Start();                           // mark
         Delay_us(320);                          // 320uS mark
         PWM1_Stop();                            // space
         if(ir_byte[1] & (1 << bitnum)){         // test bit <bitnum>
             Delay_us(1680);                     // 1680uS space for a '1'
         }
         else{
             Delay_us(680);                      // 680uS space for a '0'
         }
     }
     for(bitnum = 0; bitnum < 8; bitnum++){      // 8 bit command LSB first
         PWM1_Start();                           // mark
         Delay_us(320);                          // 320uS mark
         PWM1_Stop();                            // space
         if(ir_byte[2] & (1 << bitnum)){         // test bit <bitnum>
             Delay_us(1680);                     // 1680uS space for a '1'
         }
         else{
             Delay_us(680);                      // 680uS space for a '0'
         }
     }

     // extension message 2 bits 10
     for(bitnum = 0; bitnum < 2; bitnum++){      // 2 bit extension LSB first
         PWM1_Start();                           // mark
         Delay_us(320);                          // 320uS mark
         PWM1_Stop();                            // space
         Delay_us(1680);                         // 1680uS space for a '1'
         PWM1_Start();                           // mark
         Delay_us(680);                          // 680uS space for a '0'
         PWM1_Stop();                            // space
     }

     // inter-message gap
     PWM1_Start();                               // mark
     Delay_us(320);                              // 320uS mark
     PWM1_Stop();                                // space
     Delay_ms(40);                               // 40 millisecond inter-message gap

     // repeat 5 bit address
     for(bitnum = 0; bitnum < 5; bitnum++){      // 5 bit address LSB first
         PWM1_Start();                           // mark
         Delay_us(320);                          // 320uS mark
         PWM1_Stop();                            // space
         if(ir_byte[1] & (1 << bitnum)){         // test bit <bitnum>
             Delay_us(1680);                     // 1680uS space for a '1'
         }
         else{
             Delay_us(680);                      // 680uS space for a '0'
         }
     }

     // second message has inverted data
     for(bitnum = 0; bitnum < 8; bitnum++){      // 8 bit command LSB first
         PWM1_Start();                           // mark
         Delay_us(320);                          // 320uS mark
         PWM1_Stop();                            // space
         if(ir_byte[2] & (1 << bitnum)){         // test bit <bitnum>
             Delay_us(680);                      // 680uS space for an INVERTED '1'
         }
         else{
             Delay_us(1680);                     // 1680uS space for an INVERTED '0'
         }
     }

     // check sequence 2 bits 01 (inverted extension message)
     for(bitnum = 0; bitnum < 2; bitnum++){      // 2 bit extension LSB first
         PWM1_Start();                           // mark
         Delay_us(320);                          // 320uS mark
         PWM1_Stop();                            // space
         Delay_us(680);                          // 1680uS space for an INVERTED '1'
         PWM1_Start();                           // mark
         Delay_us(1680);                         // 680uS space for an INVERTED '0'
         PWM1_Stop();                            // space
     }

     // inter-message gap
     PWM1_Start();                               // mark
     Delay_us(320);                              // 320uS mark
     PWM1_Stop();                                // space
     Delay_ms(40);                               // 40 millisecond inter-message gap
}