// InfraRed Receiver routines
// Vishay TSOP4838 38KHz receiver Right leg = +5v, Centre leg = 0v, left leg = output
// your receiver may be different so CHECK THE DATASHEET
// use basic easyPIC7 board at 5V with PIC18F45K22 8MHz xtal + 2x16LCD + IR Rx on C0 with NO pull-up/down
// amateur code - use at your own risk - dodgy coding practices
// 38KHz receiver works fine with 40KHz Sony and 36KHz RC5 - just at half the range

// LCD routines provided to give diagnostic information
//    remove LCD library and all LCD references in your final project

//  ****** be sure to edit address and button values below the comment with the stars ******

sbit ir_in at RC0_bit;                      // Infrared receiver input
sbit ir_in_direction at TRISC0_bit;         // Infrared receiver input

// Lcd module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

// Pin direction
sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D7_Direction at TRISB3_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D4_Direction at TRISB0_bit;

// constants
#define NSAMPLES 32                // number of IR cycles to read for timing print

// global variables
char irnowm;                       // remember current RC5/RC6 mark/space status
char irwas2t;                      // RC5/RC6 have we just had a 2t event?
char irbyte;                       // assemble a byte of IR
unsigned int retint;               // general purpose integer
signed int temp_bit_s;             // bit timing store
signed int temp_bit_m;             // bit timing store
char keephex[3];                   // store hex value as ascii
unsigned char ir_byte;
unsigned char ir_result[6];
unsigned char button;

// prototypes see function for explanation
int get_mark(void);
int get_space(void);
void nec_read(void);
unsigned char get_nec_byte(void);

void main() {
    ANSELA = 0;                   // Configure PORT pins as digital
    ANSELB = 0;                   // Configure PORT pins as digital
    ANSELC = 0;                   // Configure PORT pins as digital
    ANSELD = 0;                   // Configure PORT pins as digital
    ADCON0 = 0;                   // A/D off
    ADCON1 = 0x0f;                // all digital
//     CM1CON = 7;                   // Disable 18F4620 comparator
    CM1CON0 = 0;                  // disable 18F45K22 comparator
    CM2CON0 = 0;                  // disable 18F45K22 comparator
    T0CON = 0x86;                 // timer on 16 bit prescale 128:1
    TMR0L = 96;                   // Timer0 initial value
    TMR0H = 0xff;
    INTCON = 0x00;                // Disnable interrupts

    LATA = 0;                     // all off
    LATB = 0x80;                  // disable GLCD if fitted
    LATD = 0;                     // all off
    LATC = 0;                     // all off
    LATE = 0;                     // all off
    TRISA = 0xC1;                 // all output except OSC pins
    TRISB = 0x81;                 // all output except b0, b7
    TRISC = 0x01;                 // all output except c0
    TRISD = 0x00;                 // all output
    TRISE = 0;                    // all output
    ir_in_direction = 1;          // IR input

    Delay_ms(1000);                        // Wait for LCD to stabilize
    Lcd_Init();                            // Initialize Lcd
    Lcd_Cmd(_LCD_CLEAR);                   // Clear display
    Lcd_Cmd(_LCD_CURSOR_OFF);              // Turn cursor off
    Lcd_Out(2,1, "waiting for IR  ");      // Print text to Lcd, 2nd row, 1st column
    while(1){
        ir_result[0]=0;                    // prevent trigger on IR glitch
        ir_result[1]=0;                    // prevent trigger on IR glitch
        ir_result[2]=0;                    // prevent trigger on IR glitch
        ir_result[3]=0;                    // prevent trigger on IR glitch
        nec_read();
        
        //  ******  adjust values below to suite YOUR remote codes ******
        
        if((ir_result[0]==0x10)&&(ir_result[1]==0xee)){      // ADJUST for your remote ******
            button = ir_result[2];
            switch(button){
                case 0x04: LATD.F0 = ~LATD.F0;break;   // album+ button       // ADJUST for your remote ******
                case 0x05: LATD.F1 = ~LATD.F1;break;   // playlist+ button    // ADJUST for your remote ******
                case 0x06: LATD.F2 = ~LATD.F2;break;   // chapter+ button     // ADJUST for your remote ******
                case 0x08: LATD.F3 = ~LATD.F3;break;   // album- button       // ADJUST for your remote ******
                case 0x09: LATD.F4 = ~LATD.F4;break;   // playlist- button    // ADJUST for your remote ******
                case 0x0a: LATD.F5 = ~LATD.F5;break;   // chapter- button     // ADJUST for your remote ******

                default: break;
            }
        }
        Delay_ms(500);                      // limit toggling speed
    }
    while(1);              // loop forever
}


// print unsigned char as hex
void hexout(unsigned char hexval){
int temp_hex;
char hextxt[7];

    temp_hex = hexval;                     // to integer
    IntToHex(temp_hex, hextxt);            // integer to ASCII hex
    keephex[0] = hextxt[2];                // store ascii hex for later
    keephex[1] = hextxt[3];
    keephex[2] = 0;                        // null terminate
}

// NEC protocol: 1/4 or 1/3 duty cycle 38khz (21 pulses per bit approx)
// 9ms mark then 4.5ms space    header
// 560us mark, 560us space      zero
// 560us mark, 1.69ms space     one
// 4 bytes, LSB first
// 560us mark - long space      stop bit of one
// option 1 = repeat every 110ms (if wanted), but just 9ms mark, 2.25 space, 560us mark (no data)
// option 2 = repeat full message
// 00ff = default address, custom codes available, byte 4 is inverse of byte 3
// byte1 is low byte of address, byte2 is either inverse of byte 1 or high byte

// print out NEC values - 16 bit address, 8 bit command, 8 bit inverted command
void nec_read(void){
signed int temp_nec;
char count_byte;

    Lcd_Out(1,1, "NEC protocol IR ");    // Print text to Lcd, 1st row, 1st column

        while(ir_in);                         // wait for IR to go low

        temp_nec = get_mark();
        if ((temp_nec > 500) && (temp_nec < 800)){       // expect 9mS mark
            temp_nec = get_space();
            if ((temp_nec > 200) && (temp_nec < 400)){   // expect 4.5mS space
                for(count_byte = 0 ; count_byte < 4 ; count_byte++ ){
                    ir_result[count_byte] = get_nec_byte();
                }
            }
            Lcd_Out(2,1, "NEC            ");    // Print text to Lcd, 2nd row, 1st column
            for(count_byte = 0 ; count_byte < 4 ; count_byte++){
                hexout(ir_result[count_byte]);
                Lcd_Chr(2,(count_byte * 3)+5,keephex[0]);
                Lcd_Chr(2,(count_byte * 3)+6,keephex[1]);
            }
            Delay_ms(10);             // wait for repeat messages to clear
        }
}

// get one byte of NEC
unsigned char get_nec_byte(void){
char n;
unsigned char ir_byte;
signed int temp_ir;

    for(n=0 ; n<8 ; n++){
        temp_ir = get_mark();                         // expect ~600uS
        temp_ir = get_space();                        // 500uS = 0, 1.6uS = 1
        ir_byte >>= 1;                                // shift in a zero
        if (temp_ir > 70){                            // check > 900uS
            ir_byte += 0x80;                          // change 0 to a 1 if so
        }
    }
    return ir_byte;                                   // return final byte
}

// get IR mark time (low)
int get_mark(void){
        TMR0H = 0;
        TMR0L = 0;
        while(!ir_in);                       // wait for IR to go high
        LATE.F0 = 0;                         // LED off
        retint = TMR0L;
        retint += (TMR0H<<8);
        return retint;
}

// get IR space time (high)
int get_space(void){
        TMR0H = 0;
        TMR0L = 0;
        while(ir_in);                        // wait for IR to go low
        LATE.F0 = 1;                         // LED on
        retint = TMR0L;
        retint += (TMR0H<<8);
        return retint;
}