// InfraRed Receiver routines
// Vishay TSOP4838 38KHz receiver Right leg = +5v, Centre leg = 0v, left leg = output
// your receiver may be different so CHECK THE DATASHEET
// use basic easyPIC7 board at 5V with PIC18F45K22 8MHz xtal + 2x16LCD + IR Rx on C0 with NO pull-up/down
// amateur code - use at your own risk - dodgy coding practices
// 38KHz receiver works fine with 40KHz Sony and 36KHz RC5 - just at half the range

sbit ir_in at RC0_bit;                      // Infrared receiver input
sbit ir_in_direction at TRISC0_bit;         // Infrared receiver input

// Lcd module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

// Pin direction
sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D7_Direction at TRISB3_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D4_Direction at TRISB0_bit;

// global variables
char irnowm;                       // remember current RC5/RC6 mark/space status
char irwas2t;                      // RC5/RC6 have we just had a 2t event?
char irbyte;                       // assemble a byte of IR
unsigned int retint;               // general purpose integer
signed int temp_bit_s;             // bit timing store
signed int temp_bit_m;             // bit timing store
char keephex[3];                   // store hex value as ascii
unsigned char ir_byte;
char ir_toggle;                    // toggle bit value
unsigned char ir_result[5];
unsigned char button;

// prototypes see function for explanation
int get_mark(void);
int get_space(void);
unsigned char get_rc5_byte(char nbits);
void getrc5bit(void);
void rc5_read(void);


void main() {
    ANSELA = 0;                   // Configure PORT pins as digital
    ANSELB = 0;                   // Configure PORT pins as digital
    ANSELC = 0;                   // Configure PORT pins as digital
    ANSELD = 0;                   // Configure PORT pins as digital
    ADCON0 = 0;                   // A/D off
    ADCON1 = 0x0f;                // all digital
//     CM1CON = 7;                   // Disable 18F4620 comparator
    CM1CON0 = 0;                  // disable 18F45K22 comparator
    CM2CON0 = 0;                  // disable 18F45K22 comparator
    T0CON = 0x86;                 // timer on 16 bit prescale 128:1
    TMR0L = 96;                   // Timer0 initial value
    TMR0H = 0xff;
    INTCON = 0x00;                // disable interrupts

    LATA = 0;                     // all off
    LATB = 0x80;                  // disable GLCD if fitted
    LATD = 0;                     // all off
    LATC = 0;                     // all off
    LATE = 0;                     // all off
    TRISA = 0xC1;                 // all output except OSC pins
    TRISB = 0x81;                 // all output except b0
    TRISC = 0x01;                 // all output except c0
    TRISD = 0x00;                 // all output
    TRISE = 0;                    // all output
    ir_in_direction = 1;          // IR input

    Delay_ms(1000);                // Wait for LCD to stabilize
    Lcd_Init();                            // Initialize Lcd
    Lcd_Cmd(_LCD_CLEAR);                   // Clear display
    Lcd_Cmd(_LCD_CURSOR_OFF);              // Turn cursor off
    Lcd_Out(1,1, "RC5 protocol  IR");    // Print text to Lcd, 1st row, 1st column
    Lcd_Out(2,1, "waiting for IR  ");    // Print text to Lcd, 2nd row, 1st column

    while(1){
        ir_result[0]=0;                                    // prevent false trigger on IR glitch
        ir_result[1]=0;                                    // prevent false trigger on IR glitch
        rc5_read();
        if(ir_result[0]==0x0f){                            // adjust to address of YOUR remote ******
            button = ir_result[1];
            switch (button){
                case 0x10: LATD.F0 = ~ LATD.F0; break;     // remote button vol+       // adjust value for YOUR remote ******
                case 0x11: LATD.F1 = ~ LATD.F1; break;     // remote button vol-       // adjust value for YOUR remote ******
                case 0x18: LATD.F2 = ~ LATD.F2; break;     // remote button treble+    // adjust value for YOUR remote ******
                case 0x19: LATD.F3 = ~ LATD.F3; break;     // remote button treble-    // adjust value for YOUR remote ******
                case 0x16: LATD.F4 = ~ LATD.F4; break;     // remote button bass+      // adjust value for YOUR remote ******
                case 0x17: LATD.F5 = ~ LATD.F5; break;     // remote button bass-      // adjust value for YOUR remote ******
                default:  break;
            }
            Delay_ms(500);                                 // limit rate of toggle
        }
    }
}

// print unsigned char as hex
void hexout(unsigned char hexval){
int temp_hex;
char hextxt[7];

    temp_hex = hexval;                     // to integer
    IntToHex(temp_hex, hextxt);            // integer to ASCII hex
    keephex[0] = hextxt[2];                // store ascii hex for later
    keephex[1] = hextxt[3];
    keephex[2] = 0;                        // null terminate
}

// get IR mark time (low)
int get_mark(void){
        TMR0H = 0;
        TMR0L = 0;
        while(!ir_in);                       // wait for IR to go high
        LATE.F0 = 0;                         // LED off
        retint = TMR0L;
        retint += (TMR0H<<8);
        return retint;
}

// get IR space time (high)
int get_space(void){
        TMR0H = 0;
        TMR0L = 0;
        while(ir_in);                        // wait for IR to go low
        LATE.F0 = 1;                         // LED on
        retint = TMR0L;
        retint += (TMR0H<<8);
        return retint;
}


// RC5 protocol 14 bits 36kHz 1.778ms per high/low bit,
// 24.889ms whole command
// 0=0.889ms mark, 0.889ms space       1=0.889ms space, 0.889ms mark
// start bit = 1     (only second half of bit (mark) is detectable as space is lost in no data period
// start bit = 1     (i.e. two start bits)
// toggle = x        alternates between key presses to distinguish continued press

// print out RC5 values
void rc5_read(void){
signed int temp_ir;
unsigned char rc5_byte1;
unsigned char rc5_byte2;

    rc5_byte1=0;                          // init
    rc5_byte2=0;                          // init
    while(ir_in);                         // wait for IR to go low

    irwas2t = 0;                                   // init
    irnowm = 0;                                    // init
    temp_ir = get_mark();                          // second half of start bit 1
    if ((temp_ir > 40) && (temp_ir < 80)){         // expect 890uS mark start bit1
        rc5_byte1 = get_rc5_byte(7);               // start bit2 '1' + toggle + 5 bit address
        rc5_byte2 = get_rc5_byte(6);               // 6 bit command
        Lcd_Out(2,1, "t=0 ad=   cmd=  ");          // Print text to Lcd, 2nd row, 1st column
        if (rc5_byte1 & 0x20){
            Lcd_Chr(2,3,'1');
        }
        else{
            Lcd_Chr(2,3,'0');
        }
        hexout(rc5_byte1 & 0x1F);                  // 5 bits address
        Lcd_Chr(2,8,keephex[0]);               // recover character stored by hexout function
        Lcd_Chr(2,9,keephex[1]);
        hexout(rc5_byte2);
        Lcd_Chr(2,15,keephex[0]);
        Lcd_Chr(2,16,keephex[1]);
        Delay_ms(100);                              // wait for repeat messages to clear
    }
    ir_result[0]=rc5_byte1&0x1f;                    // read only 5 address bits
    ir_result[1]=rc5_byte2&0x3f;                    // read only 6 command bits
}

// get one byte of RC5
unsigned char get_rc5_byte(char nbits){
char n;

    irbyte = 0;                                  // initialise
    for(n = 0 ; n < nbits ; n++){
        getrc5bit();
    }
    return irbyte;
}

//get a bit from RC5 protocol
void getrc5bit(void){

     if(!irnowm){
          // now a space if here
          if(!irwas2t){
              // now space after 1t mark
              irnowm = 0;                                       // will leave in space state
              temp_bit_s = get_space();                         // get 1st half of bit
              temp_bit_m = get_mark();                          // get 2nd half of bit (+ maybe half of next)
              irwas2t = 0xff;                                   // assume 2t until checked
              if(temp_bit_m < 80) irwas2t = 0;                  // if 1t
              irbyte <<= 1;                                     // rotate in a 0
              irbyte ++;                                        // change to a 1
          }
          else{
              // now space after 2t mark
              irnowm = 0xff;                                    // will leave in mark state
              temp_bit_s = get_space();                         // get 2nd half of bit (+ maybe half of next)
              irwas2t = 0xff;                                   // assume 2t until checked
              if(temp_bit_s < 80) irwas2t = 0;                  // if 1t
              irbyte <<= 1;                                     // rotate in a 0
          }
     }
     else{
          // now a mark if here
          if(!irwas2t){
              // now mark after 1t space
              irnowm = 0xff;                                    // will leave in mark state
              temp_bit_m = get_mark();                          // get 1st half of bit
              temp_bit_s = get_space();                         // get 2nd half of bit (+ maybe half of next)
              irwas2t = 0xff;                                   // assume 2t until checked
              if(temp_bit_s < 80) irwas2t = 0;                  // if 1t
              irbyte <<= 1;                                     // rotate in a 0
          }
          else{
              // now mark after 2t space
              irnowm = 0;                                       // will leave in space state
              temp_bit_m = get_mark();                          // get 2nd half of bit (+ maybe half of next)
              irwas2t = 0xff;                                   // assume 2t until checked
              if(temp_bit_m < 80) irwas2t = 0;                  // if 1t
              irbyte <<= 1;                                     // rotate in a 0
              irbyte ++;                                        // change to a 1
          }
     }
}