// InfraRed Receiver routines
// Vishay TSOP4838 38KHz receiver Right leg = +5v, Centre leg = 0v, left leg = output
// your receiver may be different so CHECK THE DATASHEET
// use basic easyPIC7 board at 5V with PIC18F45K22 8MHz xtal + 2x16LCD + IR Rx on C0 with NO pull-up/down
// press reset button to return to start screen
// amateur code - use at your own risk - dodgy coding practices
// 38KHz receiver works fine with 40KHz Sony - just at half the range
// turn on LEDs for PORTD and PORTE
// Sony TV remote control buttons 0-7 toggle PORTD LEDs 0 to 7, 8=all off, 9=all on

sbit ir_in at RC0_bit;                      // Infrared receiver input
sbit ir_in_direction at TRISC0_bit;         // Infrared receiver input

// global variables
unsigned int retint;               // general purpose integer
signed int temp_bit_s;             // bit timing store
signed int temp_bit_m;             // bit timing store
unsigned char ir_byte;
unsigned char ir_result[5];
unsigned char button;              // remote button pressed

// prototypes see function for explanation
void sony_read(void);
int get_mark(void);
int get_space(void);
unsigned char get_sony_byte(char nbits);

void main() {
    ANSELA = 0;                   // Configure PORT pins as digital
    ANSELB = 0;                   // Configure PORT pins as digital
    ANSELC = 0;                   // Configure PORT pins as digital
    ANSELD = 0;                   // Configure PORT pins as digital
    ADCON0 = 0;                   // A/D off
    ADCON1 = 0x0f;                // all digital
//    CM1CON = 7;                   // Disable 18F4620 comparator
    CM1CON0 = 0;                  // disable 18F45K22 comparator
    CM2CON0 = 0;                  // disable 18F45K22 comparator
    T0CON = 0x86;                 // timer on 16 bit prescale 128:1
    TMR0L = 96;                   // Timer0 initial value
    TMR0H = 0xff;
    INTCON = 0x00;                // Disable interrupts

    LATA = 0;                     // all off
    LATB = 0x80;                  // disable GLCD if fitted
    LATC = 0;                     // all off
    LATD = 0;                     // all off
    LATE = 0;                     // all off
    TRISA = 0xC1;                 // all output except OSC pins
    TRISB = 0x81;                 // all output except b0,b7
    TRISC = 0x01;                 // all output except c0
    TRISD = 0x00;                 // all output
    TRISE = 0;                    // all output
    ir_in_direction = 1;          // IR input

    Delay_ms(1000);                // Wait for LCD to stabilize

    while(1){
        sony_read();
        if(ir_result[1]==1){                 // Sony address 01 is TV
            button = ir_result[0];
            switch (button){
                case 9: LATD.F0 = ~LATD.F0;break;        // button TV '0'
                case 0: LATD.F1 = ~LATD.F1;break;        // button TV '1'
                case 1: LATD.F2 = ~LATD.F2;break;        // button TV '2'
                case 2: LATD.F3 = ~LATD.F3;break;        // button TV '3'
                case 3: LATD.F4 = ~LATD.F4;break;        // button TV '4'
                case 4: LATD.F5 = ~LATD.F5;break;        // button TV '5'
                case 5: LATD.F6 = ~LATD.F6;break;        // button TV '6'
                case 6: LATD.F7 = ~LATD.F7;break;        // button TV '7'
                case 7: LATD = 0x00;break;               // button TV '8'
                case 8: LATD = 0xff;break;               // button TV '9'
                default:break;
            }
        }
        delay_ms(500);                                   // limit toggle rate
    }
    while(1);              // loop forever
}

// get IR mark time (low)
int get_mark(void){
        TMR0H = 0;
        TMR0L = 0;
        while(!ir_in);                       // wait for IR to go high
        LATE.F0 = 0;                         // LED off
        retint = TMR0L;
        retint += (TMR0H<<8);
        return retint;
}

// get IR space time (high)
int get_space(void){
        TMR0H = 0;
        TMR0L = 0;
        while(ir_in);                        // wait for IR to go low
        LATE.F0 = 1;                         // LED on
        retint = TMR0L;
        retint += (TMR0H<<8);
        return retint;
}

// Sony Remote 40kHz 7 bits command (LSB first) then 5 bits address
// there are also 15 bit and 20 bit versions of Sony protocol
//
// 40kHz 7 bits command (LSB first) then 5 bits address
// 2.4ms mark=start
// 0.6sp, 0.6 mark = 0
// 0.6sp, 1.2 mark = 1
//
// tv               address=01
// vcr1             address=02
// vcr2             address=03
// radio/cd         address=04
// laser disc       address=06
// surround sound   address=0c
// tuner            address=0d
// amp              address=10 (theatre / cassette / tuner)
// cd player        address=11
// equaliser        address=12
// HDMI control     address=17
// dvd              address=1a

// print out Sony values
void sony_read(void){
signed int temp_mk;

    ir_result[0] = 0;                            // return 0, 0 if glitch
    ir_result[1] = 0;                            // return 0, 0 if glitch
    while(ir_in);                                // wait for IR to go low

    temp_mk = get_mark();
    if ((temp_mk > 120) && (temp_mk < 200)){     // expect 2.4mS mark
        ir_result[0] = get_sony_byte(7);         // 7 bit command
        ir_result[1] = get_sony_byte(5);         // 5 bit address
        Delay_ms(10);             // wait for repeat messages to clear
    }
}

// get one byte of Sony
unsigned char get_sony_byte(char nbits){
char n;
unsigned char ir_byte;
signed int temp_ir;

    ir_byte = 0;                                  // initialise
    for(n=0 ; n<nbits ; n++){
        temp_ir = get_space();                    // expect ~510uS
        temp_ir = get_mark();                     // 650uS = 0, 1.2uS = 1
        ir_byte >>= 1;
        if (temp_ir > 60){                        //
            ir_byte += 0x80;
        }
    }
    ir_byte >>= 8-nbits;                          // convert 5/7 bits to 8
    return ir_byte;
}