// simple Sony IR transmitter - give 40kHz IR output on RB3 of PIC16F628
// you may need to hold RB4 low when programming - as LVP is enabled after erase
// no libraries are needed
// select HS oscillator and fit 8MHz crystal along with 22pf x2, MCLR enable and WDT disable
// connect a clean 5V supply to PIC (I would guess 4.5V battery would do, but I have not tried it)
// fit 10k resistor from MCLR pin to +5V supply
// fit 0.1 uF decoupling capacitor across PIC supply pins as close as possible to PIC
// connect a push switch PORTB.B0 to ground, B1 to ground, B2 to ground, B3 to ground
// connect an IR LED to PORTB.B3 with 330 Ohm series resistor for short range
// longer range requires a driver circuit to give higher power for IR LED - schematic is in project folder but connect B3, not C2 as diagram wrongly shows

// prototypes
void pwm_start(void);
void pwm_stop(void);
void send_1(void);
void send_0(void);

void main() {
unsigned char sony_address;
unsigned char sony_command;
char n;

    CMCON = 7;                         // Comparators off.
    PORTA = 0;                           // all pins low
    TRISA = 0xff;                      // all input (that can be)
    TRISB = 0xf7;                       // all input  except RB3
    OPTION_REG.B7 = 0;                  // ~RBPU bit is cleared to enable weak pull-ups

    T2CON = 0x04;                       // prescaler + turn on TMR2;
    PR2 = 0x31;                         // freq 0x31 for 40kHz, 0x23 for 56 kHz
    CCPR1L = 0x17;                      // set low time 0x17 for 40kHz, 0x11 for 56kHz
    CCP1CON = 0x30;                     // duty lowest bits + PWM mode (PWM off)

    while (1){
        if((PORTB & 0x27) != 0x27){          // wait for switch to go low
            sony_address = 0;               // initialise
            sony_command = 0;               // initialise
            if (!PORTB.B0){                  // button 1
                sony_address = 1;           // TV = 1
                sony_command = 0;           // button 1
            }
            if (!PORTB.B1){                  // button 2
                sony_address = 1;           // TV = 1
                sony_command = 1;           // button 2
            }
            if (!PORTB.B2){                  // button 3
                sony_address = 1;           // TV = 1
                sony_command = 2;           // button 3
            }
            if (!PORTB.B5){                  // button 4
                sony_address = 1;           // TV = 1
                sony_command = 3;           // button 4
            }
            pwm_start();                    // 40KHz mark for 2.4mS start bit
            Delay_us(2400);
            pwm_stop();                     // space  for 1.2mS start bit
            Delay_us(1200);
            for(n=0; n<7 ; n++){            // 7 command bits
                if(sony_command & (1<<n)){  // check each bit
                    send_1();
                }
                else{
                    send_0();
                }
            }
            for(n=0; n<5 ; n++){            // 5 address bits
                if(sony_address & (1<<n)){  // check each bit
                    send_1();
                }
                else{
                    send_0();
                }
            }
            pwm_stop();                     // allow gap between messages
            Delay_ms(300);                  // limit message rate with suitable message free time
        }
    }
}

// send a sony 0 bit
void send_0(void){
        pwm_start();                    // 40KHz mark for .6 mS
        Delay_us(600);
        pwm_stop();                     // space  for .6 mS
        Delay_us(600);
}

// send a sony 1 bit
void send_1(void){
        pwm_start();                    // 40KHz mark for 1.2 mS
        Delay_us(1200);
        pwm_stop();                     // space  for .6 mS
        Delay_us(600);
}


// start PWM
void pwm_start(void){
    CCP1CON = 0x3c;
}

// stop PWM
void pwm_stop(void){
    CCP1CON = 0x30;
}