// simple Sony IR transmitter - gives 40kHz IR output on GP2 of PIC12F683
// no libraries are needed
// select internal oscillator, MCLR enable and WDT disable
// connect a clean 5V supply to PIC (I would guess 4.5V battery would do, but I have not tried it)
// fit 10k resistor from MCLR pin to +5V supply
// fit 0.1 uF decoupling capacitor across PIC supply pins as close as possible to PIC
// connect a push switch GP0 to ground, GP1 to ground, GP4 to ground, GP5 to ground
// connect an IR LED to GP2 with 330 Ohm series resistor for short range
// longer range requires a driver circuit to give higher power for IR LED
//         - EP7 driver schematic is in project folder, connect GP2 output, not C2 as in schematic

// prototypes
void pwm_start(void);                   // turn on 40KHz(mark)
void pwm_stop(void);                    // turn off 40KHz (space)
void send_1(void);                      // send a sony 0 bit of 600uS mark, 600uS space
void send_0(void);                      // send a sony 1 bit  of 1200uS mark, 600uS space

void main() {
unsigned char sony_address;             // store 5 bit address
unsigned char sony_command;             // store 7 bit command
char n;                                 // just a counter

    OSCCON = 0x71;                      // Internal 8MHz osc.

    ADCON0 = 0;                         // all pins digital
    ANSEL = 0;                          // all pins digital
    CMCON0 = 7;                         // Comparators off.
    GPIO = 0;                           // all pins low
    TRISIO = 0xfb;                      // all input, bar GP2
    WPU = 0x33;                         // weak pull-ups on button inputs
    OPTION_REG.B7 = 0;                  // ~GPPU bit is cleared to enable weak pull-ups

    T2CON = 0x04;                       // prescaler + turn on TMR2;
    PR2 = 0x31;                         // freq 0x31 for 40kHz
    CCPR1L = 0x17;                      // set low time 0x17 for 40kHz
    CCP1CON = 0x30;                     // duty lowest bits + PWM mode (PWM off)

    while (1){
        if((GPIO & 0x33) != 0x33){          // wait for switch to go low
            sony_address = 0;               // initialise
            sony_command = 0;               // initialise
            if (!GPIO.B0){                  // PIC button 1
                sony_address = 1;           // TV = 1
                sony_command = 0;           // Sony button 1
            }
            if (!GPIO.B1){                  // PIC button 2
                sony_address = 1;           // TV = 1
                sony_command = 1;           // TV button 2
            }
            if (!GPIO.B4){                  // PIC button 3
                sony_address = 1;           // TV = 1
                sony_command = 2;           // TV button 3
            }
            if (!GPIO.B5){                  // PIC button 4
                sony_address = 1;           // TV = 1
                sony_command = 3;           // TV button 4
            }
            pwm_start();                    // 40KHz mark for 2.4mS start bit
            Delay_us(2400);
            pwm_stop();                     // space  for 1.2mS start bit
            Delay_us(1200);
            for(n=0; n<7 ; n++){            // 7 command bits
                if(sony_command & (1<<n)){  // check each bit
                    send_1();
                }
                else{
                    send_0();
                }
            }
            for(n=0; n<5 ; n++){            // 5 address bits
                if(sony_address & (1<<n)){  // check each bit
                    send_1();
                }
                else{
                    send_0();
                }
            }
            pwm_stop();                     // allow gap between messages
            Delay_ms(300);                  // limit message rate with suitable message free time
        }
    }
}

// send a sony 0 bit of 600uS mark, 600uS space
void send_0(void){
        pwm_start();                    // 40KHz mark for .6 mS
        Delay_us(600);
        pwm_stop();                     // space  for .6 mS
        Delay_us(600);
}

// send a sony 1 bit  of 1200uS mark, 600uS space
void send_1(void){
        pwm_start();                    // 40KHz mark for 1.2 mS
        Delay_us(1200);
        pwm_stop();                     // space  for .6 mS
        Delay_us(600);
}


// start PWM
void pwm_start(void){
    CCP1CON = 0x3c;
}

// stop PWM
void pwm_stop(void){
    CCP1CON = 0x30;
}