// simple 1200 baud IR transmitter - gives 38kHz IR output on GP2 of PIC12F683
// no libraries are needed
// select internal oscillator, MCLR enable and WDT disable
// connect a clean 5V supply to PIC (I would guess 4.5V battery would do, but I have not tried it)
// fit 10k resistor from MCLR pin to +5V supply
// fit 0.1 uF decoupling capacitor across PIC supply pins as close as possible to PIC
// connect a push switch GP0 to ground, GP1 to ground, GP4 to ground, GP5 to ground
// connect an IR LED to GP2 with 330 Ohm series resistor for short range
// longer range requires a driver circuit to give higher power for IR LED
//         - EP7 driver schematic is in project folder, connect GP2 output, not C2 as in schematic

// prototypes
void pwm_start(void);                   // turn on 38KHz(mark)
void pwm_stop(void);                    // turn off 38KHz (space)
void chout(unsigned char ch_to_send);   // send a byte
void send_1(void);                      // send a 0 bit
void send_0(void);                      // send a 1 bit

void main() {
char n;                                 // just a counter

    OSCCON = 0x71;                      // Internal 8MHz osc.

    ADCON0 = 0;                         // all pins digital
    ANSEL = 0;                          // all pins digital
    CMCON0 = 7;                         // Comparators off.
    GPIO = 0;                           // all pins low
    TRISIO = 0xfb;                      // all input, bar GP2
    WPU = 0x33;                         // weak pull-ups on button inputs
    OPTION_REG.B7 = 0;                  // ~GPPU bit is cleared to enable weak pull-ups

    T2CON = 0x04;                       // prescaler + turn on TMR2;
    PR2 = 0x33;                         // freq 0x33 for 38kHz
    CCPR1L = 0x18;                      // set low time 0x18 for 38kHz
    CCP1CON = 0x30;                     // duty lowest bits + PWM mode (PWM off)

    while (1){
        if((GPIO & 0x33) != 0x33){          // wait for switch to go low
            if (!GPIO.B0){                  // PIC button 1
                chout('A');                 // send serial
                chout('a');                 // send serial
            }
            if (!GPIO.B1){                  // PIC button 2
                chout('B');                 // send serial
                chout('b');                 // send serial
            }
            if (!GPIO.B4){                  // PIC button 3
                chout('C');                 // send serial
                chout('c');                 // send serial
            }
            if (!GPIO.B5){                  // PIC button 4
                chout('D');                 // send serial
                chout('d');                 // send serial
            }
            pwm_stop();                     // allow gap between messages
            Delay_ms(300);                  // limit message rate with suitable message free time
        }
    }
}

// send a single serial byte 1200 baud n,8,1
void chout(unsigned char ch_to_send){
char n;                                // just a counter
unsigned char temp_char;

    temp_char = ch_to_send;
    send_0();                          // start bit

    for(n=0 ; n<8 ; n++){              // count 8 bits
        if (temp_char & 1){
           send_1();
        }
        else{
           send_0();
        }
        temp_char = temp_char >> 1;
    }
    
    send_1();                          // stop bit
}

// send a 0 bit of 833uS mark
void send_0(void){
        pwm_start();                    // 38KHz mark for 833 uS
        Delay_us(833);
}

// send a 1 bit  of 833uS space
void send_1(void){
        pwm_stop();                     // space  for 833 uS
        Delay_us(833);
}


// start PWM
void pwm_start(void){
    CCP1CON = 0x3c;
}

// stop PWM
void pwm_stop(void){
    CCP1CON = 0x30;
}